#!/usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-07-31 18:05:54
LastEditors:  
LastEditTime: 2019-07-31 18:05:54
Description:  
"""

# GAS = 2000000               # 默认的gas值
# GAS_PRICE = 22000000000     # 默认的gas_price值
GAS = 100000               # 默认的gas值
GAS_PRICE = 100000     # 默认的gas_price值
PROVIDER = 'http://localhost:7545'
