#! /usr/bin/env python
# -*- coding: utf8 -*-
"""
Author:       yuyongpeng@hotmail.com
Github:       https://github.com/yuyongpeng/
Date:         2019-04-10 08:51:15
LastEditors:  
LastEditTime: 2019-04-10 08:51:15
Description:  
"""

class kk:
    def __init__(self):
        pass
    def ts(self):
        print("tsts")